﻿Mediasite.namespace("Player.ConvivaConfiguration");

Mediasite.Player.ConvivaConfiguration.ConvivaConfigurationFactory = function () {
    var self = this;

    self.getPartialConvivaConfiguration = function (playerContext) {

        if (playerContext.GlobalOptions.EnableConviva) {
            return {
                isEnabled: playerContext.GlobalOptions.EnableConviva,
                sourceUrl: Mediasite.Player.Configurations.conviva.sourceUrl,
                serviceUrl: playerContext.GlobalOptions.ConvivaServiceUrl,
                customerId: playerContext.GlobalOptions.ConvivaCustomerId,
                title: playerContext.CurrentPresentation.Title,
                isLive: playerContext.CurrentPresentation.IsLivePresentation(),
                presentationId: playerContext.CurrentPresentation.PresentationId,
                playerType: playerContext.PlayerLayoutOptions.PlayerType,
                duration: playerContext.CurrentPresentation.Duration / 1000,
                recordedDateString: playerContext.CurrentPresentation.Date,
                numberOfSlides: (function () {
                    var slideStream = undefined;
                    slideStream = playerContext.CurrentPresentation.GetStream(Mediasite.ContentStreamType.Slide);
                    if (slideStream != undefined && slideStream.HasSlides() && !playerContext.CurrentPresentation.IsLivePresentation()) {
                        return slideStream.Slides.length;
                    } else {
                        return 0;
                    }
                })(),
                numberOfVideos: (function () {
                    var videoCount = 0;
                    for (var streamCount = 0; streamCount < playerContext.CurrentPresentation.Streams.length; streamCount++) {
                        if (playerContext.CurrentPresentation.Streams[streamCount].HasVideo()) {
                            videoCount++;
                        }
                    }
                    return videoCount;
                })(),
                playerId: playerContext.GlobalOptions.PlayerId,
                layoutId: playerContext.GlobalOptions.LayoutId,
                siteName: playerContext.GlobalOptions.SiteName,
                siteVersion: playerContext.GlobalOptions.SiteVersion,
                siteBuildNumber: playerContext.GlobalOptions.SiteBuildNumber,
                pluginMimeType: playerContext.pluginMimeType,
                playerName: playerContext.GlobalOptions.PlayerName,
                userNameHash: playerContext.GlobalOptions.UserNameHash,
                multisiteId: playerContext.GlobalOptions.MultisiteId,
                streamType: "",
                pluginMimeType: "",
                streamUrl: ""
            };
        } else {
            return { isEnabled: false };
        }
    }

    self.makeGetConvivaConfiguration = function(partialConfiguration) {
        return function(lateBoundConfig) {
            var extendedConfig = $.extend({}, partialConfiguration);
            extendedConfig.streamType = lateBoundConfig.streamType;
            extendedConfig.streamUrl = lateBoundConfig.streamUrl;
            extendedConfig.pluginMimeType = lateBoundConfig.pluginMimeType;
            extendedConfig.mediaPlayerType = lateBoundConfig.mediaPlayerType;

            return extendedConfig;
        }
    }
}


Mediasite.Player.ConvivaConfiguration.FlashVarsGenerator = function (configurationValues) {
    var self = this;
    var metadata = {};
    var getConfigValue = function (propertyName) {
        return configurationValues[propertyName];
    }

    this.getFlashVars = function () {
        return {            
            Conviva_namespace: encodeURIComponent(getConfigValue("sourceUrl")),
            Conviva_serviceUrl: encodeURIComponent(getConfigValue("serviceUrl")),
            Conviva_customerId: encodeURIComponent(getConfigValue("customerId")),
            Conviva_videolevelJSON: encodeURIComponent(JSON.stringify(generateMetadataObject()))
        };
    }

    function generateMetadataObject() {
        metadata["assetName"] = asssetName();
        metadata["isLive"] = getConfigValue("isLive");
        metadata["playerName"] = Mediasite.Player.MediaPlayerType.GetDisplayName(parseInt(configurationValues["mediaPlayerType"]));
        metadata["viewerId"] = getConfigValue("userNameHash");

        metadata["tags"] = {
            presentationId: getConfigValue("presentationId"),
            playerType: getPlayerTypeName(configurationValues["playerType"]),
            duration: getConfigValue("duration"),
            recordedDate: getDateFormatted(configurationValues["recordedDateString"]),
            site: location.host,
            referrer: document.referrer,
            streamProtocol: getStreamProtocol(configurationValues["pluginMimeType"]),
            numberOfSlides: getConfigValue("numberOfSlides"),
            numberOfVideos: getConfigValue("numberOfVideos"),
            catalogId: Mediasite.GetQueryParam("catalog"),
            playerVersion: playerVersion(),
            playerId: "[" + getConfigValue("playerId").substring(0, 6) + "...] " + getConfigValue("playerName"),
            siteName: getConfigValue("siteName"),
            streamUrl: getConfigValue("streamUrl"),
            multisiteId: getConfigValue("multisiteId")
        };

        var parentObject = {};
        parentObject["src"] = getConfigValue("sourceUrl");
        parentObject["metadata"] = metadata;

        return parentObject;
    }

    function asssetName() {
        return "[" + getConfigValue("presentationId").substring(0, 6) + "...] " + getConfigValue("title") + " [" + getStreamTypeName(getConfigValue("streamType")) + "]";
    }

    function playerVersion() {
        return getConfigValue("siteVersion") + "." + getConfigValue("siteBuildNumber");
    }

    function getStreamTypeName(contentStreamType) {
        if (contentStreamType == 0) {
            return 'Video 1';
        } else if (contentStreamType == 2) {
            return 'Slide';
        } else if (contentStreamType == 3) {
            return 'Presentation';
        } else if (contentStreamType == 4) {
            return 'Video 2';
        } else if (contentStreamType == 5) {
            return 'Video 3';
        }
    }

    function asssetName() {
        return "[" + getConfigValue("presentationId").substring(0, 6) + "...] " + getConfigValue("title") + " [" + getStreamTypeName(getConfigValue("streamType")) + "]";
    }

    function playerVersion() {
        return getConfigValue("siteVersion") + "." + getConfigValue("siteBuildNumber");
    }

    function getStreamTypeName(contentStreamType) {
        if (contentStreamType == 0) {
            return 'Video 1';
        } else if (contentStreamType == 2) {
            return 'Slide';
        } else if (contentStreamType == 3) {
            return 'Presentation';
        } else if (contentStreamType == 4) {
            return 'Video 2';
        } else if (contentStreamType == 5) {
            return 'Video 3';
        }
    }

    function getDateFormatted(dateString) {
        var date = new Date(dateString);

        return getMonthName(date.getMonth()) + " " + date.getDate() + ", " + date.getFullYear()
    }

    function getMonthName(month) {
        var monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

        return monthNames[month];
    }

    function getPlayerTypeName(playerType) {

        if (playerType == Mediasite.Player.PlayerType.None) {
            return 'None';
        }
        if (playerType == Mediasite.Player.PlayerType.NewStandardPlayer) {
            return 'Full Experience';
        }
        if (playerType == Mediasite.Player.PlayerType.PipPlayer) {
            return 'Pip';
        }
        if (playerType == Mediasite.Player.PlayerType.MiniPlayer) {
            return 'Mini';
        }
        if (playerType == Mediasite.Player.PlayerType.StandardPlayer) {
            return 'Standard';
        }
        if (playerType == Mediasite.Player.PlayerType.ScreenReader) {
            return 'Screen Reader';
        }
        if (playerType == Mediasite.Player.PlayerType.ContentOnlyPlayer) {
            return 'Content Only';
        }
        if (playerType == Mediasite.Player.PlayerType.SmartPlayer) {
            return 'Multiview';
        }
    }

    function getStreamProtocol(mimeType) {
        if (mimeType == "video/mp4") {
            return "MP4";
        } else if (mimeType == "video/x-mp4-fragmented") {
            return "Smooth Streaming";
        } else {
            return "";
        }
    }

};
