(function ($, Filters) {
    var debug = false;
    // debug = true;

    var canvasSupport = !!window.HTMLCanvasElement && !Mediasite.BrowserDetect.Android(); // Android doesn't render the blur well

    function BlurBackground() { }


    BlurBackground.prototype.options = {
        src: '',
        ratio: undefined,   // natural size / css px size
        width: undefined,   // image's dimensions in css px (no unit)
        height: undefined,
        padding: 10, // whitespace around image in css px  (no unit)
        blurRadius: 6,  // in css px (no unit); will be multipled by image's ratio when generating blur
        doubleBackground: true,
        elementClass: 'ui-blur-background',
        blurContainerClass: 'ui-blur-image',
        noCanvasClass: 'no-canvas',
        foregroundClass: 'foreground',
        backgroundClass: 'background',
        doubleBackgroundClass: 'double-background'
    };

    BlurBackground.prototype._create = function () {
        var self = this;

        this._super();
        this._fillOptionsFromCss();

        this._populateElements();
        this._applyBaseStyle();

        var image = this.sourceImage = new Image();
        this.sourceImage.onload = function () {
            clearTimeout(self.sourceImage.fakeOnLoad);
            self._onLoadImage();
        };
    };

    BlurBackground.prototype._fillOptionsFromCss = function() {
        if (!this.options.width) {
            // Get element's CSS-enforced size for setting dimensions/ratio on image later

            this.options.width = this.element.width();
            this.options.height = this.element.height();

            // This is a little fragile, since it depends on this.element currently having CSS applied to it
            // which determines the image's dimensions
        }

        if (!this.options.src) {
            var background = this.element.css('background-image').match(/^url\(['"]?([^"|^']+)['"]?\)$/);
            if (background) {
                this.element.css('background-image', 'none');
                this.options.src = background[1];
            }
        }
    };

    BlurBackground.prototype._init = function () {
        var self = this;
        
        // If image is already loaded, then onload may not fire
        this.sourceImage.fakeOnLoad = setTimeout(function () {
            if (self.sourceImage.width) self._onLoadImage();
        }, 1);

        self.sourceImage.src = this.options.src;
    };

    BlurBackground.prototype._populateElements = function () {
        var self = this;
        var ele = this.element;
        var options = this.options;

        var container = this.blurContainer = $("<span />").addClass(this.options.blurContainerClass);
        this.blurContainer = container;
        this.element.prepend(container);

        var image = $("<img />").addClass(this.options.foregroundClass);
        this.image = image;
        image.attr('src', this.options.src);
        image.appendTo(container);

        var background = $("<img />")
			.addClass(this.options.backgroundClass);
        if (this.options.doubleBackground) {
            var backgroundDouble = background.clone()
            	.addClass(this.options.doubleBackgroundClass)
            	.appendTo(this.element);
            background = background.add(backgroundDouble);
        }
        this.background = background;

        background.appendTo(container);        
    };

    BlurBackground.prototype._applyBaseStyle = function () {
        var self = this;

        this.element.addClass(this.options.elementClass);
        this.element.css({
            'position': 'relative',
            'border': 'none',
            'background': 'none'
        });

        this.background.hide();

        this._applyStyle();
    };


    BlurBackground.prototype._applyStyle = function() {
        this.blurContainer.css({
            width: this.options.width + 'px',
            height: this.options.height + 'px'
        });

        var padding = (0 - this.options.padding) + 'px';
        var width = (this.options.width + this.options.padding * 2) + 'px';
        var height = (this.options.height + this.options.padding * 2) + 'px';

        this.background.css({
            'position': 'absolute',
            'top': padding,
            'left': padding,
            'right': padding,
            'bottom': padding,
            'width': width,
            'height': height
        });
    };

    BlurBackground.prototype._onLoadImage = function () {

        if (!this.options.ratio) {
            var displayWidth = this.options.width;
            var naturalWidth = this.sourceImage.width;
            this.options.ratio = naturalWidth / displayWidth || 1;
            debug && window.console && console.log(["Ratio:", this.options.ratio, '=', naturalWidth, '/', displayWidth, '--', this.options.src].join(" "));
        }

        this._setBlurBackground();
    };

    BlurBackground.prototype._setBlurBackground = function () {
        var self = this;

        var backgroundBlur;
        try {
            var dimensions = { padding: this.options.padding * this.options.ratio };
            backgroundBlur = this._generateBlur(this.sourceImage, dimensions);
        } catch (e) {
            // If drawing the canvas throws weird errors  regarding dimensions, cannot find pixels, etc.
            // then make sure either the ratio is set or the dimensions make sense

            if (debug && window.console) {
                console.warn(e);
                console.error(e.stack);
            }
        }


        if (backgroundBlur) {
            this.background.attr('src', backgroundBlur)
                .show();
        } else {
            this.element.addClass(this.options.noCanvasClass);
            this.background.hide();
        }
    };

    BlurBackground.prototype._generateBlur = function (image, dimensions) {
        if (!canvasSupport) return;
        var filtered = this._stackBlur(image, dimensions);

        var canvas = document.createElement('canvas');
        canvas.width = filtered.imageData.width;
        canvas.height = filtered.imageData.height;

        var context = canvas.getContext('2d');
        context.scale(this.options.ratio, this.options.ratio);

        context.putImageData(filtered.imageData, 0, 0);

        var dataUrl = canvas.toDataURL();
        return dataUrl;
    };

    BlurBackground.prototype._stackBlur = function (image, dimensions) {
        var filter = Mediasite.Filters.stackBlur;
        var radius = this.options.blurRadius * this.options.ratio;

        var data = Mediasite.Filters.filterImage(filter, image, dimensions, radius);
        return data;
    };

    // #endregion




    Filters.BlurBackground = BlurBackground;
} (jQuery, Mediasite.Filters));




$.widget('ui.clickableButton', $.ui.button, {
    options: {
        click: $.noop
    },
    _create: function () {
        this._super();
        var self = this;

        this.element.on('click', function (e) { self.options.click(e); });

        // A minimal .enableTouch() implementation for $.ui.button
        this.element.on('touchend', function (e) {
            setTimeout(function () {
                self.element.trigger('mouseleave');
            }, 500);
        });

        this._setTitle();

        this.initialized = true;
    },

    _setOption: function (key, value) {
        this._super(key, value);
        if (this.initialized) {
            if (key == "label" || key == "title") this._setTitle();
        }
    },

    _setTitle: function () {
        var title = this.options.title !== false
            ? this.options.title || this.options.label
            : ''
        if (title) {
            this.element.attr('title', title);
        } else {
            this.element.removeAttr('title');
        }
    }
});



$.widget('ui.singleImageButton', $.ui.clickableButton, $.extend(
    {
        _resetButton: function () {
            this._super();
            // Preserve BlurBackground elements on reset
            if (this.blurContainer) this.element.append(this.blurContainer);
        }
    },
    new Mediasite.Filters.BlurBackground));

