﻿/// <reference path="../../Scripts/jquery.enableTouch.js"/>


(function ($, window, undefined) {
    $.widget('ui.timelineSlider', $.ui.slider, {
        options: {
            _locked: {},
            // Slide preview
            getSlidesAt: null,
            // Slide preview widget options
            defaultSlideImage: null,
            slideAspectRatios: null,
            formatTime: null,
            // Markers
            markers: [], // item: { Time: number in ms, Text: string }
            markerNearnessTolerance: 0.005, // factor of slider.max
            markerMinimumNearnessTolerance: 1500,
            // Events
            slidePreview: $.noop,
            hidePreview: $.noop
        },
        _init: function () {
            var self = this;
            this.initialized = true;

            this._initSlidePreview();
            this._initCurrentTime();
            this._initMarkers();
            this._initTouchArea();
            this.element.watchLinger({
                events: {
                    lingerchange: $.proxy(this.onSlide, this),
                    lingerend: $.proxy(this.onSlideStop, this),
                    start: 'timelinesliderstart timelineslider:start',
                    change: 'timelinesliderslide timelineslider:slide',
                    end: 'timelinesliderstop timelineslider:stop'
                }
            })
        },
        _initSlidePreview: function () {
            if (!this.slidePreview) {
                this.slidePreview = this.element.find(".preview");
                this.slidePreview.slidePreview();
            }
            var options = {};
            for (var key in $.ui.slidePreview.prototype.options) {
                if (!$.ui.slidePreview.prototype.options.hasOwnProperty(key)) continue;

                if (this.options[key]) options[key] = this.options[key];
            }
            
            this.slidePreview.slidePreview("option", options);
        },
        _initCurrentTime: function () {
            if (!this.currentTime) {
                this.currentTime = this.element.find(".preview-time");
                this.currentTime.currentTimePreview();
            }

            var options = {};
            for (var key in $.ui.currentTimePreview.prototype.options) {
                if (!$.ui.currentTimePreview.prototype.options.hasOwnProperty(key)) continue;

                if (this.options[key]) options[key] = this.options[key];
            }
            
            this.currentTime.currentTimePreview("option", options);
        },
        _initMarkers: function () {
            this._sortMarkers();
            this.markers = this.element.find('.markers');

            this.layoutMarkers();


            if (!this.markerPreview) {
                this.markerPreview = this.element.find('.markerPreview');
                this.markerPreview.markerPreview();
            }
        },
        _sortMarkers: function() {
            this.options.markers = this.options.markers.sort(function(a,b) {
                return a.time - b.time;
            });
        },
        _initTouchArea: function () {
            this.element
                .watchLinger({
                    selector: '.toucharea, .thumb',
                    events: {
                        'lingerchange': this.onMouseMove.bind(this),
                        'lingerend': this.onMouseOut.bind(this),
                        'end': 'mouseleave touchend adjacenttouchend'
                    }
                })
                .enableTouch();
        },

        _setOption: function (key, value) {
             if (key in this.options._locked) {
                this._setLockedOption(key, value);
                return;
            }

            var rangeChanged = (key === "max" && this.options.max.value != value)
                || (key === "min" && this.options.min.value != value);

            this._super(key, value);

            if (rangeChanged || key == "markers") {
                this._sortMarkers();
                this.layoutMarkers();
            }
            if (this.slidePreview && $.ui.slidePreview.prototype.options.hasOwnProperty(key)) {
                this.slidePreview.slidePreview("option", key, value);
            }
        },
        _setLockedOption: function(key, value) {
            this.options._locked[key] = value;
        },
        _getLockedOption: function(key) {
            if (key in this.options._locked) {
                return this.options._locked[key];
            } else {
                return this.options[key];
            }
        },
        lockOption: function(key) {
            if (key in this.options._locked) return;
            this._setLockedOption(key, this.options[key]);
        },
        unlockOption: function (key) {
            var value = this._getLockedOption(key);
            delete this.options._locked[key];
            this._setOptions(key, value);
        },
        
        onSlide: function(event) {
            this._lingeringSliding = true;

            this.lockOption('max');
            this.showPreview(event, event.data.value, true);
        },

        onSlideStop: function(event) {
            this._lingeringSliding = false;

            this.unlockOption('max');
            this.hidePreview();
        },

        onMouseMove: function (event) {
            if (this._lingeringSliding) {
                event.data.reset();
                return;
             }

            this.lockOption('max');
            var enlargedSlide = event.data && event.data.extended;
            this.showPreview(event, undefined, enlargedSlide);
        },

        onMouseOut: function () {
            // window.console && console.log(event.type, event);
            if (this._lingeringSliding) {
                return;
            }
            
            this.unlockOption('max');
            this.hidePreview();
        },

        showPreview: function (positionOn, sliderValue, enlargedSlide) {
            var self = this;

            if (sliderValue === void 0) {
                positionOn = positionOn || this.handle;
                sliderValue = this.getSliderValueFromPosition(positionOn);
            }
            
            if (enlargedSlide) {
                this.slidePreview.slidePreview("hide", true);
                self.showCurrentTime(sliderValue);
                self.showEnlargedSlidePreview(sliderValue);
            } else {
                this._trigger('hidePreview');
                this.hideCurrentTime();
                self.showSlidePreview(sliderValue);
            }

            this.showMarkerPreview(sliderValue);
        },
        hidePreview: function () {
            this.slidePreview.slidePreview("hide");
            this.hideCurrentTime();
            this.markerPreview.markerPreview("hide");
            this._trigger('hidePreview');
        },
        hideCurrentTime: function() {
            this.currentTime.currentTimePreview("hide", true);
        },

        showSlidePreview: function (time) {
            if (this.options.disabled) return;

            var images = this.options.getSlidesAt(time);
            if (!(images && images.length)) { 
                this.slidePreview.slidePreview("hide"); 
                this.showCurrentTime(time);
            } else {
                var positionAt = 'left+' + (time / this.options.max * 100) + '%' + ' center';
                var urls = $.map(images, function (image, index) { return image.url; });
                this.slidePreview.slidePreview("show", this.element, { images: urls, time: time }, positionAt);
                this.hideCurrentTime();
            }
        },

        showEnlargedSlidePreview: function (sliderValue) {
            if (this.options.disabled) return;

            this._trigger('slidePreview', null, { time: sliderValue });
        },

        showCurrentTime: function (time) {
            if (this.options.disabled) return;
            
            var positionAt = 'left+' + (time / this.options.max * 100) + '%' + ' center';
            this.currentTime.currentTimePreview("show", this.element, {  time: time }, positionAt);
        },


        showMarkerPreview: function (time) {
            this.currentMarkerElement && this.currentMarkerElement.removeClass('highlighted');
            var marker = this.getMarkerNearest(time);
            if (marker) {
                this.currentMarkerElement = this.markerElements[marker.time];
                this.currentMarkerElement && this.currentMarkerElement.addClass('highlighted');
                
                var colors = this._getMarkerColors(marker) || false;
                this.markerPreview.markerPreview("option", "colors", colors);
                this.markerPreview.markerPreview("show", this.currentMarkerElement, marker);
            } else {
                this.markerPreview.markerPreview("hide", true);
            }
        },


        getSliderValueFromPosition: function (positionOn) {
            var left;

            var e = positionOn;
            while (e && typeof e.pageX === "undefined") {
                e = e.event || e.originalEvent;
            }

            if (e && typeof e.pageX !== "undefined") {
                var sliderLeft = this.element.offset().left;
                left = e.pageX - sliderLeft;
            } else if (typeof positionOn.offsetX !== "undefined") {
                left = positionOn.offsetX;
            }


            var width = this.element.width();

            left = Math.min(Math.max(0, left), width);

            var relativePosition = left / width;
            var sliderValue = this.options.max * relativePosition;

            return sliderValue;
        },


        layoutMarkers: function () {
            this.markers.empty();
            this.markerElements = {};
            if (!(this.options.markers && this.options.markers.length)) return;

            for (var i = 0, marker; marker = this.options.markers[i]; i++) {
                var position = (marker.time / this.options.max * 100) + '%';
                
                var colors = this._getMarkerColors(marker);

                var markerElement = $("<div />", { "class": 'marker' }).appendTo(this.markers);
                markerElement.css('left', position);
                if (colors) {
                    markerElement.css({
                        'border-top-color': colors.dark,
                        'border-bottom-color': colors.dark,
                        'background-color': colors.normal
                    });
                }

                this.markerElements[marker.time] = markerElement;
            }
        },

        getMarkerNearest: function (time) {
            if (!(this.options.markers && this.options.markers.length)) return;
            var tolerance = Math.max((this.options.max * this.options.markerNearnessTolerance),
                this.options.markerMinimumNearnessTolerance);

            var markers = this.getMarkersNear(time);
            var marker =
                !markers.after || markers.before && (time - markers.before.time) < (markers.after.time - time)
                ? markers.before
                : markers.after

            if (marker && Math.abs(marker.time - time) < tolerance) {
                return marker;
            }

        },

        getMarkersNear: function (time) {
            if (!(this.options.markers && this.options.markers.length)) return;
            for (var i = 0, marker; marker = this.options.markers[i]; i++) {
                if (marker.time >= time) {
                    break;
                }
            }

            return {
                before: this.options.markers[i - 1],
                after: this.options.markers[i]
            }
        },

        _getMarkerColors: function (marker) {
            if (!marker.color) return;
            if (!this._markerColors) this._markerColors = {};
            if (!this._markerColors[marker.color]) {
                this._markerColors[marker.color] = this._generateMarkerColors(marker.color);
            }
            
            return this._markerColors[marker.color];
        },
        _generateMarkerColors: function (color) {
            return {
                dark: Mediasite.Color.darken(color),
                normal: Mediasite.Color.toHexString(color),
                light: Mediasite.Color.lighten(color)
            };
        }

    });

    $.widget('ui.callout', {
        options: {
            fadeIn: 'fast',
            callout: '.callout'
        },
        _init: function () {
            this.callout = this.element.find(this.options.callout);

            this.element.hide();
            this.initialized = true;
            this.options._locked = {};
        },
        _setOption: function (key, value) {
            this._super(key, value);
            if (this.initialized) {
                this.populate();
            }
        },
        show: function (positionOn, value, positionAt) {
            if (typeof value != "undefined") {
                this.options.value = value;
            }
            if (!this.initialized) return;
            value = this.options.value;

            var populated = this.populate(value);
            if (populated !== false) {
                if (!this._visible) this.reveal();
                while (positionOn.originalEvent) {
                    positionOn = positionOn.originalEvent;
                }

                this.position(positionOn, positionAt, this._visible);
            } else {
                this.hide();
            }
        },
        hide: function (instant) {
            if (instant) {
                this.element.hide();
            } else {
                this.element
                    .stop(true, true)
                    .fadeOut('fast');
            }

            this._visible = false;
        },
        reveal: function () {
            this.element
                .stop(true, true)
                .fadeIn(this.options.fadeIn);
            

            this._visible = true;
        },
        position: function (positionOn, positionAt, keepVisible) {
            if (keepVisible === void 0) {
                keepVisible = positionAt;
                positionAt = void 0;
            }
            var self = this;
            var elements = this.element.add(this.callout);

            elements
                .show()
                .position({
                    my: "center",
                    at: positionAt !== void 0 ? positionAt : "center",
                    of: positionOn,
                    collision: 'fit',
                    using: function (offset, ui) {
                        ui.element.element.css({ left: offset.left });
                    }
                });
            if (!keepVisible) { 
                elements.hide(); 
            }
        },
        populate: $.noop  // must return truthy value for callout to show
    });

    $.widget('ui.slidePreview', $.ui.callout, {
        options: {
            callout: '.thumb',
            defaultSlideImage: '', // image URL
            defaultSlideAspectRatio: '16x9', // '16x9' or '4x3'
            slideAspectRatios: [ ],
            classes: {
                '4x3': 'preview-4x3',
                '16x9': 'preview-16x9',
                'image': 'image'
            },
            formatTime: function (ms) {
                var minutes = Math.floor(ms / 1000 / 60);
                var seconds = Math.floor(ms / 1000 % 60);
                var display = minutes + ':' + (seconds < 10 ? '0' : '') + seconds;
                return display;
            }
        },
        _create: function () {
            this._super();

            this.time = this.element.find('.time');
            this.imageContainer = this.element.find('.images');

            this._updateAspectRatio();

        },
        _setOption: function(key, value) {
            this._super(key, value);
            if (key == 'slideAspectRatio') { this._updateAspectRatio(); }
        },
        populate: function (value) {
            if (!this.initialized || !value) return;

            var time = value.time;

            var images = value.images;
            if (images) {
                this._showImages(images);
            } else if (this.options.defaultSlideImage) {
                this._showImages();
            } else {
                this._hideImages();
            }   

            var formattedTime = this.options.formatTime(time);
            this.time.text(formattedTime);
        },

        _updateAspectRatio: function() {
            this._populateImages(0, true);
            this._showImages();
        },

        _showImages: function (urls) {
            this.urls = urls = urls || this.urls || [];
            this._populateImages(urls.length);

            for (var i = 0, length = this.images.length; i < length; i++) {
                var imageElem = this.images[i];

                var url = urls[i] || this.options.defaultSlideImage;
                imageElem.fittedImage('src', url);
            }

            this.imageContainer.show();
        },
        _hideImages: function() {
            this.imageContainer.hide();
        },
        _populateImages: function (minLength, refresh) {
            var numImages = Math.max(minLength, this.options.slideAspectRatios.length);

            if (!this.images || this.images.length < numImages || refresh) {
                this.imageContainer.empty();
                this.images = [];

                for (var i = 0; i < numImages; i++) {
                    var aspectRatio = this.options.slideAspectRatios[i];
                    var aspectRatioClass = this.options.classes[aspectRatio] || this.options.classes[this.options.defaultSlideAspectRatio];

                    var image = $("<div><img></div>").appendTo(this.imageContainer);

                    image.addClass(this.options.classes.image);
                    image.addClass(aspectRatioClass);

                    var imageElem = image.find('img');
                    imageElem.fittedImage(image);

                    this.images.push(imageElem);
                }
            }

            return this.images.length;
        },

        position: function() {
            this._superApply(arguments);

            var widths = this.imageContainer.children().map(function(index, element) { return $(element).outerWidth(true); });
            var widthSum = 0; $.each(widths, function (index, width) { widthSum += width; });

            this.imageContainer.css('width', widthSum + 'px');


            var images = this.imageContainer.children();
            if (images.length % 2 == 0) {
                this.time.position({
                    my: "right bottom",
                    at: "right bottom",
                    of: images[images.length / 2 - 1],
                    using: function (offset, ui) {
                        ui.element.element.css('left', offset.left);
                    }
                });
            }
        }



    });

    $.widget('ui.markerPreview', $.ui.callout, {
        options: {
            'colors': {}
        },
        _init: function () {
            this._super();
            this.text = this.element.find('.text');
        },
        _setOption: function(key, value) {
            this._super(key, value);

            if (key == 'colors') {
                if (!this.options.colors) this.options.colors = {};
                this._updateColors();
            }
        },
        populate: function (value) {
            if (!this.initialized || !value) return;

            if (value && value.text) {
                this.text.text(value.text);
            } else {
                return false;
            }
        },
        _updateColors: function() {
            var color = this.options.colors.light || '';

            this.element.css({
                'border-color': color,
                'background-color': color
            });
            this.element.find('.callout').css({
                'border-bottom-color': color
            });
        }
    });

    $.widget('ui.currentTimePreview', $.ui.callout, {
        options: {
            fadeIn: 0,
            callout: '.thumb',
            formatTime: function (ms) {
                var minutes = Math.floor(ms / 1000 / 60);
                var seconds = Math.floor(ms / 1000 % 60);
                var display = minutes + ':' + (seconds < 10 ? '0' : '') + seconds;
                return display;
            }
        },
        _create: function () {
            this._super();

            this.time = this.element.find('.time');

        },
        populate: function (value) {
            if (!this.initialized || !value) return;

            var time = value.time;
            var formattedTime = this.options.formatTime(time);
            this.time.text(formattedTime);
        }
    });
})(jQuery, window);
